Class: Wasabi::Parser::MessageRef
- Inherits:
-
Struct
- Object
- Struct
- Wasabi::Parser::MessageRef
- Defined in:
- lib/wasabi/parser.rb
Overview
A wsdl message an operation input, output, or fault resolves to. Contains the message type name and its namespace identifier.
Instance Attribute Summary collapse
-
#namespace_id ⇒ Object
Returns the value of attribute namespace_id.
-
#type ⇒ Object
Returns the value of attribute type.
Instance Attribute Details
#namespace_id ⇒ Object
Returns the value of attribute namespace_id
19 20 21 |
# File 'lib/wasabi/parser.rb', line 19 def namespace_id @namespace_id end |
#type ⇒ Object
Returns the value of attribute type
19 20 21 |
# File 'lib/wasabi/parser.rb', line 19 def type @type end |